package net.p0f.samples.rhpam_fuse_integration.cdi;

import java.util.logging.Logger;

import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;

import org.jboss.resteasy.client.jaxrs.BasicAuthentication;
import org.jboss.resteasy.client.jaxrs.ResteasyWebTarget;

import net.p0f.samples.rhpam_fuse_integration.rhpam.RhpamEndpoints;
import net.p0f.samples.rhpam_fuse_integration.rhpam.invoke.ApiClient;

@ApplicationScoped
public class WebTargetProducer {
	private static final Logger log = Logger.getLogger(WebTargetProducer.class.getName());

	@Inject
	@EndpointUrl
	private String rhpamEndpoint;
	
	@Inject
	@AuthUsername
	private String rhpamUser;
	
	@Inject
	@AuthPassword
	private String rhpamPassword;

	@Produces
	@RhpamClient
	public WebTarget getWebTarget() {
		Client rc = ClientBuilder.newClient().register(new BasicAuthentication(rhpamUser, rhpamPassword));

		WebTarget wt = rc.target(rhpamEndpoint);
		
		log.info("Producing WebTarget instance for " + rhpamEndpoint);
		
		return wt;
	}

	@Produces
	@RhpamProxy
	public RhpamEndpoints getProxyClient() {
		Client rc = ClientBuilder.newClient().register(new BasicAuthentication(rhpamUser, rhpamPassword));

		ResteasyWebTarget wt = (ResteasyWebTarget)rc.target(rhpamEndpoint);

		RhpamEndpoints re = wt.proxy(RhpamEndpoints.class);

		log.info("Producing RhpamEndpoints instance for " + rhpamEndpoint);

		return re;
	}
	
	@Produces
	@SwaggerClient
	public ApiClient getApiClient() {
		ApiClient ac = new ApiClient();

		ac.setBasePath(rhpamEndpoint.replaceFirst("/server$", ""));
		ac.setUsername(rhpamUser);
		ac.setPassword(rhpamPassword);
		
		log.info("Producing ApiClient for " + ac.getBasePath());

		return ac;
	}
}
