package net.p0f.samples.rhpam_fuse_integration.rest_client;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import net.p0f.samples.rhpam_fuse_integration.cdi.RhpamClient;

// Only two endpoints here, for demonstration: /start and /signal; they expect
// their respective operands as path parameters and request payload as POST
// method body (json, of course).
//
// Probably best used with the pam-simple-process.StartStopStart process model.

@RequestScoped
@Path("/manual")
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
public class ManualInvocationClient {
	private static final Logger log = Logger.getLogger(ManualInvocationClient.class.getName());

	// This uses a CDI producer in WebTargetProducer, which in turn uses
	// another producer in ConfigProducer to load rhpam-rest-client.properties
	// and configure WebTarget with base URL and the Client with auth settings.
	@Inject
	@RhpamClient
	private WebTarget wt;

	@POST
	@Path("/start/{kieContainer}/{processModel}")
	public ManualRhpamResponse startProcess(@PathParam("kieContainer") String containerName,
								@PathParam("processModel") String modelName,
								Object data) {
		log.log(Level.INFO, "Starting a new process from " + containerName + "/" + modelName);

		WebTarget it = this.wt.path("containers/{kieContainer}/processes/{processModel}/instances")
								.resolveTemplate("kieContainer", containerName)
								.resolveTemplate("processModel", modelName);

		Response r = it.request().post(Entity.json(data));

		ManualRhpamResponse rr = new ManualRhpamResponse(r);
		
		log.log(Level.INFO, "Got response: " + rr.toString());

		return rr;
	}

	@POST
	@Path("/signal/{kieContainer}/{instanceId}/{signalName}")
	public ManualRhpamResponse signalInstance(@PathParam("kieContainer") String containerName,
								@PathParam("instanceId") int instanceId,
								@PathParam("signalName") String signalName,
								Object data) {
		log.log(Level.INFO, "Signaling instance " + containerName + "/" + instanceId + " with " + signalName);

		WebTarget it = this.wt.path("containers/{kieContainer}/processes/instances/{instanceId}/signal/{signalName}")
								.resolveTemplate("kieContainer", containerName)
								.resolveTemplate("instanceId", instanceId)
								.resolveTemplate("signalName", signalName);

		Response r = it.request().post(Entity.json(data));

		ManualRhpamResponse rr = new ManualRhpamResponse(r);
		
		log.log(Level.INFO, "Got response: " + rr.toString());

		return rr;
	}
}
