package net.p0f.samples.rhpam_fuse_integration.rest_client;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import net.p0f.samples.rhpam_fuse_integration.cdi.RhpamProxy;
import net.p0f.samples.rhpam_fuse_integration.rhpam.RhpamEndpoints;

// Same two endpoints as in the other InvocationClients: /start and /signal
//
// Exactly the same behaviour, except this time using a RestEasy proxy instead
// of a manually chauffeured WebTarget or a Swagger-generated client.
//
// This does not depend on anything but the RhpamEndpoints interface.
//
// Probably best used with the pam-simple-process.StartStopStart process model.

@RequestScoped
@Path("/proxy")
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
public class ProxyInvocationClient {
	private static final Logger log = Logger.getLogger(ProxyInvocationClient.class.getName());

	// This uses a CDI producer in WebTargetProducer, which in turn uses
	// another producer in ConfigProducer to load rhpam-rest-client.properties
	// and configure WebTarget with base URL and the Client with auth settings.
	@Inject
	@RhpamProxy
	private RhpamEndpoints re;

	@POST
	@Path("/start/{kieContainer}/{processModel}")
	public String startProcess(@PathParam("kieContainer") String containerName,
								@PathParam("processModel") String modelName,
								Object data) {
		log.log(Level.INFO, "Starting a new process from " + containerName + "/" + modelName);
		
		String newId = this.re.startProcess(containerName, modelName, data);

		log.log(Level.INFO, "New process ID: " + newId);

		return newId;
	}
	
	@POST
	@Path("/signal/{kieContainer}/{instanceId}/{signalName}")
	public void signalInstance(@PathParam("kieContainer") String containerName,
								@PathParam("instanceId") int instanceId,
								@PathParam("signalName") String signalName,
								Object data) {
		log.log(Level.INFO, "Signaling instance " + containerName + "/" + instanceId + " with " + signalName);

		this.re.signalInstance(containerName, instanceId, signalName, data);
	}
}
