package net.p0f.samples.rhpam_fuse_integration.rest_client;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import net.p0f.samples.rhpam_fuse_integration.cdi.SwaggerClient;
import net.p0f.samples.rhpam_fuse_integration.rhpam.api.ProcessInstancesApi;
import net.p0f.samples.rhpam_fuse_integration.rhpam.invoke.ApiClient;
import net.p0f.samples.rhpam_fuse_integration.rhpam.invoke.ApiException;

// Same two endpoints as in the other InvocationClients: /start and /signal
//
// Exactly the same behaviour, except this time using a Swagger proxy instead
// of a manually chauffeured WebTarget or a RestEasy proxy.
//
// All the code in net.p0f.samples.rhpam_fuse_integration.rhpam was generated
// from swagger.json (using swagger-codegen-maven-plugin, see pom.xml). Only
// the minimum number of classes was then added to this project.
//
// NOTE: ApiClient had to be modified to add HttpBasicAuth to Authentications.
//
// NOTE: ProcessInstancesApi had to be modified to activate basicAuth in all calls.
//
// Probably best used with the pam-simple-process.StartStopStart process model.

@RequestScoped
@Path("/swagger")
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
public class SwaggerInvocationClient {
	private static final Logger log = Logger.getLogger(SwaggerInvocationClient.class.getName());

	// This uses a CDI producer in WebTargetProducer, which in turn uses
	// another producer in ConfigProducer to load rhpam-rest-client.properties
	// and configure WebTarget with base URL and the Client with auth settings.
	@Inject
	@SwaggerClient
	private ApiClient ac;

	@POST
	@Path("/start/{kieContainer}/{processModel}")
	public String startProcess(@PathParam("kieContainer") String containerName,
								@PathParam("processModel") String modelName,
								Object data) {
		log.log(Level.INFO, "Starting a new process from " + containerName + "/" + modelName);

		ProcessInstancesApi pia = new ProcessInstancesApi(ac);

		String newId = null;

		try {
			newId = pia.startProcess(containerName, modelName, (data != null ? data.toString() : null)).toString();
		} catch (ApiException ae) {
			log.log(Level.SEVERE, "API problem while attempting to start process instance.", ae);
		}

		log.log(Level.INFO, "New process ID: " + newId);

		return newId;
	}
	
	@POST
	@Path("/signal/{kieContainer}/{instanceId}/{signalName}")
	public void signalInstance(@PathParam("kieContainer") String containerName,
								@PathParam("instanceId") int instanceId,
								@PathParam("signalName") String signalName,
								Object data) {
		log.log(Level.INFO, "Signaling instance " + containerName + "/" + instanceId + " with " + signalName);

		ProcessInstancesApi pia = new ProcessInstancesApi(ac);

		try {
			pia.signalProcessInstance(containerName, Integer.toUnsignedLong(instanceId), signalName, (data != null ? data.toString() : null));
		} catch (ApiException ae) {
			log.log(Level.SEVERE, "API problem while attempting to signal process instance.", ae);
		}
	}
}
